//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#include "serial.h"

//---------------------------------------------------------------------------
#pragma package(smart_init)

static HANDLE Handle = INVALID_HANDLE_VALUE;
static COMMTIMEOUTS OldComTimeOut={0};
static COMMTIMEOUTS NewComTimeOut={0};


////////////////////////////////////////////////////////////////////////////////
// SerialPortIsOpen
////////////////////////////////////////////////////////////////////////////////
bool SerialPortIsOpen(void)
{
  return (Handle!=INVALID_HANDLE_VALUE);
}

////////////////////////////////////////////////////////////////////////////////
// OpenSerialPort
////////////////////////////////////////////////////////////////////////////////
int OpenSerialPort(const char *pPort, const char *pParameter)
{
  DCB dcb;
  CloseSerialPort();
  Handle=CreateFile(pPort,GENERIC_READ | GENERIC_WRITE,0,0,OPEN_EXISTING,0,0);
  if(Handle==INVALID_HANDLE_VALUE) return false;

  GetCommTimeouts(Handle,&OldComTimeOut);
  NewComTimeOut.ReadTotalTimeoutConstant = 100;
  NewComTimeOut.ReadTotalTimeoutMultiplier = 0;
  NewComTimeOut.WriteTotalTimeoutMultiplier = 0;
  NewComTimeOut.WriteTotalTimeoutConstant = 0;
  SetCommTimeouts(Handle, &NewComTimeOut);

  dcb.DCBlength = sizeof(DCB);
  GetCommState(Handle, &dcb);
  BuildCommDCB(pParameter, &dcb);
  SetCommState(Handle, &dcb);

  FlushSerialBuffer();
  return true;
}


////////////////////////////////////////////////////////////////////////////////
// GetInQueue
////////////////////////////////////////////////////////////////////////////////
DWord GetInQueue(void)
{
COMSTAT ComStat;
DWord Errors;
  if (Handle != INVALID_HANDLE_VALUE)
  {
    ClearCommError(Handle,&Errors,&ComStat);
    return ComStat.cbInQue;
  }
  return 0;
}


////////////////////////////////////////////////////////////////////////////////
// CloseSerialPort
////////////////////////////////////////////////////////////////////////////////
int CloseSerialPort(void)
{
  if (Handle!=INVALID_HANDLE_VALUE)
  {
    SetCommTimeouts(Handle, &OldComTimeOut);
    CloseHandle(Handle);
    Handle=INVALID_HANDLE_VALUE;
    return true;
  }
  return false;
}


////////////////////////////////////////////////////////////////////////////////
// ReadSerial
////////////////////////////////////////////////////////////////////////////////
unsigned short ReadSerial(unsigned char *pBuffer, unsigned short  BytesToRead)
{
DWORD BytesAreRead=0;
  if (Handle!=INVALID_HANDLE_VALUE)
  {
    ReadFile(Handle, pBuffer, BytesToRead, &BytesAreRead, NULL);
  }
  return (unsigned short)BytesAreRead;
}


////////////////////////////////////////////////////////////////////////////////
// WriteSerial
////////////////////////////////////////////////////////////////////////////////
unsigned short  WriteSerial(unsigned char *pBuffer, unsigned short  BytesToWrite)
{
DWORD BytesAreWritten=0;
  if (Handle!=INVALID_HANDLE_VALUE)
  {
    WriteFile(Handle, pBuffer, BytesToWrite, &BytesAreWritten, NULL);
  }
  return BytesAreWritten;
}


////////////////////////////////////////////////////////////////////////////////
// FlushSerialBuffer
////////////////////////////////////////////////////////////////////////////////
void FlushSerialBuffer(void)
{
DWORD InQue;
unsigned char Trash[256];
  if (Handle!=INVALID_HANDLE_VALUE)
  {
    InQue=GetInQueue();
    while (InQue)
    {
      if (InQue>256)
      {
        ReadSerial(Trash,256);
        InQue=InQue-256;
      }
      else
      {
        ReadSerial(Trash,InQue);
        InQue=0;
      }
    }
  }
}




