{
 Engelmann & Schrader GmbH (c) 2006 www.engelmann-schrader.de

 Die Programme dienen ausschliesslich der Information. Es gibt
 weder eine Gewaehrleistung noch wird irgendeine Eigenschaft
 zugesichert. Die Verantwortung fuer den Einsatz von Programm
 bzw. Quelltext liegt vollstaendig beim Anwender.
}
UNIT RS232 ;
INTERFACE

USES Param ;

FUNCTION  Getchar : BYTE ;
PROCEDURE Putchar (C : BYTE) ;
PROCEDURE Putstring (S:STRING) ;
FUNCTION  Getcharecho:BYTE ;
FUNCTION  Charinbuffer:BOOLEAN ;
PROCEDURE NewSpeed ( BitProS : LongInt ) ;
PROCEDURE FlushBuffer ;
PROCEDURE StopIt ;
PROCEDURE Init ;
PROCEDURE Done ;

IMPLEMENTATION
USES DOS ;

CONST
  Com             : WORD = $03F8;     (* default: 1. serial interface *)
  RS232InterNr    : BYTE = $0C;       (* default: interrupt no $0C    *)

  (* interface address interrupt no
     Com1      $03F8   $0C
     Com2      $02F8   $0B
     Com3      $03E8   $0C
     Com4      $02E8   $0B
  *)

  Maxbuffer = 4096 ;

VAR
  Buffer    : ARRAY [0..Maxbuffer-1] OF Byte ;

  ZIn       : BYTE ;       (* points to next input character     *)
  ZOut      : BYTE ;       (* points to next output character    *)

  VektorOld : POINTER ;    (* saves the current interrupt vector *)
  PortOld   : BYTE ;       (* saves the current port value       *)

CONST
  THRE = $20 ;  (* Transmitter Holding Register Empty *)
  DR   = $01 ;  (* Data Ready                         *)

VAR
  C : CHAR ;

(* only if there is a new character the function returns TRUE *)
FUNCTION Charinbuffer:BOOLEAN ;
BEGIN
  Charinbuffer := ZIn <> ZOut ;
END ;

(* every time a character is being received the interrupt procedure Receive is called *)
PROCEDURE Receive ; INTERRUPT ;
BEGIN
  buffer[ZIn] := Port[Com] ;    (* stores the new character in buffer   *)

  Port[$20] := $20 ;            (* finishs the interrupt                *)

  INC (ZIn) ;                   (* increases the pointer to the buffer  *)
  IF ZIn > Maxbuffer THEN
    ZIn := 0 ;
END ;

(* initialization of the RS232 interface *)
PROCEDURE RS232Init (Com:WORD ; Baud:LongInt ; DatenBits:BYTE ;  StopBits:BYTE; Paritaet:CHAR) ;
VAR
  BaudWert : WORD ;
  Z        : BYTE ;

BEGIN
  BaudWert := LongInt(115200) DIV Baud ; (* speed           *)
  Port[Com + 3] := $80     ;             (* enable latch    *)
  Port[Com    ] := Lo (BaudWert) ;
  Port[Com + 1] := Hi (BaudWert) ;
  Port[Com + 3] := ((DatenBits - 5) AND $0003)
                      +
                    (BYTE (StopBits = 2) * 4)
                     +
                    (BYTE (Paritaet = UpCase ('E')) * $18)
                     +
                    (BYTE (Paritaet = UpCase('O'))  * $08) ;


  Port[Com + 1] := 1  ;             (* activates interrupts              *)
  Port[Com + 4] := 8 ;              (* activates RTS & DTS, OUT2         *)
  Z             := Port[Com];       (* reads pending character           *)
END ;


PROCEDURE NewSpeed ( BitProS : LongInt ) ;
BEGIN
  RS232Init (COM, BitProS, 8, 1, 'N') ;
END ;

(**)
PROCEDURE StopIt ;
BEGIN
  SetIntVec (RS232InterNr, VektorOld) ; (* restores interrupt vector  *)
  Port[$21] := PortOld ;                (* restores port value        *)
  WRITELN ('RS232: TIMEOUT!') ;
  Halt (0) ;
END ;

(* reads a character and returns it *)
FUNCTION Getchar : BYTE ;
VAR
  C       : BYTE ;
  hour    : WORD ;
  minute  : WORD ;
  second1 : WORD ;
  second2 : WORD ;
  milli  : WORD ;
BEGIN

  gettime (hour, minute, second1, milli) ;
  second2 := second1 ;
  WHILE (ZIn = ZOut) DO
    BEGIN
      IF ( ABS (second1 - Second2) < 3 ) THEN
         gettime (hour, minute, second1, milli)
      ELSE
        StopIt ;
    END  ;

  C := buffer[ZOut] ;

  INC (ZOut) ;

  IF ZOut > Maxbuffer THEN
    ZOut := 0 ;

  Getchar := C ;
END ;

(**)
PROCEDURE Putchar (C : BYTE) ;
BEGIN
  WHILE (Port[Com+5] AND THRE) = 0 DO
    ;
  Port[Com] := BYTE (C) ;
END ;

FUNCTION Getcharecho:BYTE ;
VAR
  C : BYTE ;
BEGIN
  C := Getchar ;
  Putchar (C) ;
  Getcharecho := c ;
END ;

PROCEDURE Putstring (S:STRING) ;
VAR
  I : WORD ;
BEGIN
  FOR I := 1 TO Length(S) DO
    BEGIN
      Putchar ( BYTE (S[I]) ) ;
    END ;
END ;

PROCEDURE FlushBuffer ;
BEGIN
  ZIn := 0 ;
  ZOut := 0 ;
END ;

PROCEDURE Init ;
BEGIN

  IF acominterface      = '/2' THEN
    BEGIN
      COM := MEMW [$40:2] ;
      RS232InterNr  := $0B;
    END
  ELSE IF acominterface = '/3' THEN
    BEGIN
      COM := MEMW [$40:4] ;
      RS232InterNr  := $0C;
    END
  ELSE IF acominterface = '/4' THEN
    BEGIN
      COM := MEMW [$40:6] ;
      RS232InterNr  := $0B;
    END ;

  RS232Init (COM, 9600, 8, 1, 'N') ;

  ZIn := 0 ;
  ZOut := 0 ;

  GetIntVec (RS232InterNr, VektorOld) ;
  SetIntVec (RS232InterNr, @Receive) ;
  PortOld    := Port[$21] ;
  Port [$21] := Port[$21] AND $E7 ;
END ;

PROCEDURE Done ;
BEGIN
  SetIntVec (RS232InterNr, VektorOld) ;
  Port[$21] := PortOld ;
END ;

END.