//---------------------------------------------------------------------------
#ifndef double
#define double
//---------------------------------------------------------------------------

// FFS commands
#define FORMAT                              0x01
#define OPEN                                0x02
#define CLOSE                               0x03
#define CREATE_DIRECTORY                    0x04
#define CHANGE_DIRECTORY                    0x05
#define READ                                0x06
#define WRITE                               0x07
#define SEEK                                0x08
#define RENAME                              0x09
#define REMOVE                              0x0A
#define DIR                                 0x0B

// bitmasken FFS
#define READ_ONLY_FLAG                      (1 << 0)
#define HIDDEN_FLAG                         (1 << 1)
#define SYSTEM_FLAG                         (1 << 2)
#define VOLUME_NAME_FLAG                    (1 << 3)
#define DIRECTORY_FLAG                      (1 << 4)
#define ACHIEVE_FLAG                        (1 << 5)
#define LONG_FILENAME_ATTRIBUTE             (READ_ONLY_FLAG | HIDDEN_FLAG | SYSTEM_FLAG | VOLUME_NAME_FLAG)


// USB commands
#define WRITE_ENABLE                        0x21
#define WRITE_DISABLE                       0x22
#define CONNECT                             0x23
#define DISCONNECT                          0x24
#define CARD_AVAILABLE                      0x25
#define CARD_NOT_AVAILABLE                  0x26

// MMC
#define MOUNT                               0x41
#define UNMOUNT                             0x42
#define GET_CARD_STATUS                     0x43

// bitmasken MMC
#define CARD_INSERTED			    (1 << 0)
#define CARD_WRITE_PROTECT                  (1 << 1)

// DISPLAY
#define INIT_DISPLAY                        0x61
#define CLEAR_DISPLAY                       0x62
//#define SHOW_FILE                           0x63
#define DISPLAY_WRITE                       0x64
#define DISPLAY_ONOFF                       0x65

// Cursorpositionen fr bekannte Displaytypen 1 X 16, 2 X 16, 4 X 20
#define CURSOR_1X16_LINE1_ROW1              0x00
#define CURSOR_2X16_LINE1_ROW1              0x00
#define CURSOR_2X16_LINE2_ROW1              0x40
#define CURSOR_4X20_LINE1_ROW1              0x00
#define CURSOR_4X20_LINE2_ROW1              0x40
#define CURSOR_4X20_LINE3_ROW1              0x14
#define CURSOR_4X20_LINE4_ROW1              0x54

// UPDATE
#define UPDATE                              0x81

// BAUDRATE
#define BAUDRATE                            0xA1
#define PING                                0xA2

// IO
#define IO_SET_DIRECTION                    0xC1
#define IO_SET_PULLUP                       0xC2
#define IO_GET_INPUT                        0xC3
#define IO_SET_OUTPUT                       0xC4

//---------------------------------------------------------------------------


//---------------------------------------------------------------------------

typedef unsigned char uint8_t;
typedef unsigned short uint16_t;
typedef unsigned int uint32_t;

typedef struct
{
  uint8_t Second;
  uint8_t Hour;
  uint8_t Minute;
  uint8_t Day;
  uint8_t Month;
  uint16_t Year;
}
TimeAndDate_t;

typedef enum
{
  FILEOPENMODE_INVALID=0,
  FILEOPENMODE_READONLY,
  FILEOPENMODE_WRITE,
  FILEOPENMODE_APPEND,
}
FileOpenMode_t;

typedef enum
{
  WHENCE_SEEKSET=0,
  WHENCE_SEEKCUR,
  WHENCE_SEEKEND
}
Whence_t;

typedef struct
{
  uint8_t   FileName[8];
  uint8_t   FileExtension[3];
  uint8_t   Attribute;
  uint8_t   Reserved;
  uint8_t   CreationTimeMilliSeconds;
  uint16_t  CreationTime;
  uint16_t  CreationDate;
  uint16_t  LastAccessDate;
  uint8_t   ReservedForFat32[2];
  uint16_t  LastWriteTime;
  uint16_t  LastWriteDate;
  uint16_t  StartingCluster;
  uint32_t  FileSizeInBytes;
}
DirectoryEntryLayout_t;

//---------------------------------------------------------------------------

extern uint8_t Mount_(void);
extern uint8_t UnMount_(void);
extern uint8_t GetCardStatus_(void);
extern uint8_t FileOpen_(uint8_t Handle, uint8_t *pName, uint8_t FileOpenMode,TimeAndDate_t TimeAndDate);
extern uint8_t FileClose_(uint8_t Handle,TimeAndDate_t TimeAndDate);
extern uint16_t FileRead_(uint8_t Handle, uint8_t *pData, uint16_t Count, uint16_t Size);
extern uint16_t FileWrite_(uint8_t Handle, uint8_t *pData, uint16_t Count, uint16_t Size);
extern uint8_t FileSeek_(uint8_t Handle, uint16_t Position, uint8_t Whence);
extern uint16_t Dir_(uint8_t *pData);
extern uint8_t ChangeDirectory_(uint8_t *pName);
extern uint8_t CreateDirectory_(uint8_t *pName,TimeAndDate_t TimeAndDate);
extern uint8_t RenameFile_(uint8_t *pOldName,uint8_t *pNewName,TimeAndDate_t TimeAndDate);
extern uint8_t RemoveFile_(uint8_t *pName);
extern uint8_t QuickFormat_(uint8_t *pName);

extern uint8_t InitDisplay_(void);
extern uint8_t ClearDisplay_(void);
extern uint8_t DisplayWrite_(uint8_t CursorPosition, uint8_t Count, uint8_t *pText);
extern uint8_t DisplayOnOff_(uint8_t OnOff);

extern uint8_t Connect_(void);
extern uint8_t DisConnect_(void);
extern uint8_t WriteDisable_(void);
extern uint8_t WriteEnable_(void);
extern uint8_t CardAvailable_(void);
extern uint8_t CardNotAvailable_(void);

extern uint8_t Baudrate_(uint32_t BaudRate);
extern uint8_t Ping_(void);

extern uint8_t SetDirection_(uint16_t Direction);
extern uint8_t SetPullUp_(uint16_t PullUp);
extern uint8_t SetOutput_(uint16_t Output);
extern uint8_t GetInput_(uint16_t *Data);

//---------------------------------------------------------------------------

#endif
