//---------------------------------------------------------------------------

#include "double.h"
#include "serial.h"
#include <string.h>
#include <ctype.h>

//---------------------------------------------------------------------------

idata uint8_t CmdBuffer[64]={0};


////////////////////////////////////////////////////////////////////////////////
// MsDelay
// Wartezeit in Millisekunden
////////////////////////////////////////////////////////////////////////////////
void MsDelay(uint16_t DelayInMilliSeconds)
{
 uint16_t i;

 #ifdef SIMULATOR 
   DelayInMilliSeconds *= 60 ;  // Nur wenn mit dem Simulator gearbeitet wird.
 #endif  

 while ( DelayInMilliSeconds )
  {
   for ( i = 113; i != 0; --i)	// 8051 @ 11 MHz
    ;
   --DelayInMilliSeconds ;
  }

}


///////////////////////////////////////////////////////////////////////////////
// SendCommand
// bertrgt einen Befehl zum Doublestick
// Der Befehl kann bis zu 4 Parameter haben. Nicht vorhandene Parameter werden
// ausgelassen.
//
// Befehlsaufbau:
// Byte 0 Befehl
// Byte 1 LowByte Gesamtlnge
// Byte 2 HighByte Gesamtlnge
//
// LowByte Lnge Parameter 1
// HighByte Lnge Parameter 1
// Parameter 1 (1 - 512 byte)

// ...

// LowByte Lnge Parameter 4
// HighByte Lnge Parameter 4
// Parameter 4 (1 - 512 byte)

// Byte n: 8 Bit XOR Checksumme ber gesamten Befehl

///////////////////////////////////////////////////////////////////////////////
void SendCommand( void )
{
uint8_t XOR=0;
uint16_t Count;

  FlushSerialBuffer();

  Count=(uint16_t)(CmdBuffer[1]+256*CmdBuffer[2]);
  Count--;
  while (Count--)
    XOR=XOR^CmdBuffer[Count];
  Count=(uint16_t)(CmdBuffer[1]+256*CmdBuffer[2]);
  CmdBuffer[Count-1]=XOR;

  WriteSerial(CmdBuffer, Count);
}

///////////////////////////////////////////////////////////////////////////////
// GetResult
// Wartet maximal 10 Sekunden auf eine Antwort.
// Der Doublestick bricht auch eine Befehlsequenz nach 10 Sekunden ab.
// Das Ergebnis ist gltig wenn die XOR Checksumme der Antwort stimmt.
///////////////////////////////////////////////////////////////////////////////
uint8_t GetResult(uint16_t TimeOut)
{
  uint8_t Result[3];

  while ( TimeOut )
  {
   if (GetInQueue() >= 3)
    {
      ReadSerial(Result,3);
      if (((Result[0] ^ Result[1])== Result[2]) && Result[1]!=0)
        return Result[1];
      else
        return 0 ;
    }
   MsDelay ( 1 ) ;
   --TimeOut ;   
  }
  return 0 ;
}

///////////////////////////////////////////////////////////////////////////////
// Mount_
// Datentrger mounten
// Die Karte wird neu initialisiert. Das Filesystem eingelesen.
///////////////////////////////////////////////////////////////////////////////
uint8_t Mount_(void)
{
uint8_t x=0;
  // befehl
  CmdBuffer[x++]=MOUNT;
  // platzhalter fr gesamtlnge. wird als letzes eingetragen
  CmdBuffer[x++]=0;
  CmdBuffer[x++]=0;
  // plus 1 byte fr xor checksumme
  x++;
  // berechnete lnge eintragen
  CmdBuffer[1]=(uint8_t)(x % 256);
  CmdBuffer[2]=(uint8_t)(x / 256);
  // checksumme berechnen und befehl senden
  SendCommand();
  // warte maximal STANDARD_TIMEOUT auf antwort
  return GetResult(STANDARD_TIMEOUT);
}


///////////////////////////////////////////////////////////////////////////////
// UnMount_
// Datentrger unmounten
// Geffnete Dateien werden geschlossen. Karte kann danach entnommen werden.
///////////////////////////////////////////////////////////////////////////////
uint8_t UnMount_(void)
{
uint8_t x=0;
  CmdBuffer[x++]=UNMOUNT;
  // platzhalter fr gesamtlnge. wird als letzes eingetragen
  CmdBuffer[x++]=0;
  CmdBuffer[x++]=0;
  // plus 1 byte fr xor checksumme
  x++;
  CmdBuffer[1]=(uint8_t)(x % 256);
  CmdBuffer[2]=(uint8_t)(x / 256);
  SendCommand();
  return GetResult(STANDARD_TIMEOUT);
}

///////////////////////////////////////////////////////////////////////////////
// FileOpen_
// ffnet die Datei mit dem Namen pName
// Handle: [0..3]
// pName: 8.3 Name von A..Z, 0..9, #, &, %, (, ), -, @
// FileOpenMode: FILEOPENMODE_READONLY, FILEOPENMODE_WRITE, FILEOPENMODE_APPEND
///////////////////////////////////////////////////////////////////////////////
uint8_t FileOpen_(uint8_t Handle, uint8_t *pName, uint8_t FileOpenMode,TimeAndDate_t TimeAndDate)
{
uint8_t x=0;
uint16_t p;
uint8_t Name8Dot3[11];
  memset(Name8Dot3,' ',sizeof(Name8Dot3));

  // Name in einen gltigen 8.3 Name ohne Punkt! z.B. ABCDEFGHTXT
  for (p=0; p<11; p++)
  {
    if (*pName=='.')
    {
      p=7;
      pName++;
    }
    else
      Name8Dot3[p]=(char)toupper(*pName++);
  }

  // command
  CmdBuffer[x++]=OPEN;
  // platzhalter fr gesamtlnge. wird als letzes eingetragen
  CmdBuffer[x++]=0;
  CmdBuffer[x++]=0;
  // handle laenge
  CmdBuffer[x++]=1;
  CmdBuffer[x++]=0;
  // handle
  CmdBuffer[x++]=Handle;
  // laenge filename
  CmdBuffer[x++]=11;
  CmdBuffer[x++]=0;
  // filename
  CmdBuffer[x++]=Name8Dot3[0];
  CmdBuffer[x++]=Name8Dot3[1];
  CmdBuffer[x++]=Name8Dot3[2];
  CmdBuffer[x++]=Name8Dot3[3];
  CmdBuffer[x++]=Name8Dot3[4];
  CmdBuffer[x++]=Name8Dot3[5];
  CmdBuffer[x++]=Name8Dot3[6];
  CmdBuffer[x++]=Name8Dot3[7];
  CmdBuffer[x++]=Name8Dot3[8];
  CmdBuffer[x++]=Name8Dot3[9];
  CmdBuffer[x++]=Name8Dot3[10];
  // laenge filemode
  CmdBuffer[x++]=1;
  CmdBuffer[x++]=0;
  CmdBuffer[x++]=FileOpenMode;

  // laenge time and date
  CmdBuffer[x++]=7;
  CmdBuffer[x++]=0;
  CmdBuffer[x++]=TimeAndDate.Second;
  CmdBuffer[x++]=TimeAndDate.Minute;
  CmdBuffer[x++]=TimeAndDate.Hour;
  CmdBuffer[x++]=TimeAndDate.Day;
  CmdBuffer[x++]=TimeAndDate.Month;
  CmdBuffer[x++]=(uint8_t)(TimeAndDate.Year % 256);
  CmdBuffer[x++]=(uint8_t)(TimeAndDate.Year / 256);

  // plus 1 byte fr xor checksumme
  x++;
  CmdBuffer[1]=(uint8_t)(x % 256);
  CmdBuffer[2]=(uint8_t)(x / 256);
  SendCommand();
  return GetResult(STANDARD_TIMEOUT);
}


///////////////////////////////////////////////////////////////////////////////
// FileClose_
// Handle: [0..3]
///////////////////////////////////////////////////////////////////////////////
uint8_t FileClose_(uint8_t Handle,TimeAndDate_t TimeAndDate)
{
uint8_t x=0;
  CmdBuffer[x++]=CLOSE;
  CmdBuffer[x++]=0;
  CmdBuffer[x++]=0;
  // handle laenge
  CmdBuffer[x++]=1;
  CmdBuffer[x++]=0;
  // handle
  CmdBuffer[x++]=Handle;

  // laenge time and date
  CmdBuffer[x++]=7;
  CmdBuffer[x++]=0;
  CmdBuffer[x++]=TimeAndDate.Second;
  CmdBuffer[x++]=TimeAndDate.Minute;
  CmdBuffer[x++]=TimeAndDate.Hour;
  CmdBuffer[x++]=TimeAndDate.Day;
  CmdBuffer[x++]=TimeAndDate.Month;
  CmdBuffer[x++]=(uint8_t)(TimeAndDate.Year % 256);
  CmdBuffer[x++]=(uint8_t)(TimeAndDate.Year / 256);

  x++;
  CmdBuffer[1]=(uint8_t)(x % 256);
  CmdBuffer[2]=(uint8_t)(x / 256);
  SendCommand();
  return GetResult(STANDARD_TIMEOUT);
}


///////////////////////////////////////////////////////////////////////////////
// FileWrite_
///////////////////////////////////////////////////////////////////////////////
uint16_t FileWrite_(uint8_t Handle, uint8_t *pData, uint16_t Count, uint16_t Size)
{
uint8_t x=0;
uint16_t y;
  CmdBuffer[x++]=WRITE;
  CmdBuffer[x++]=0;
  CmdBuffer[x++]=0;
  // handle laenge
  CmdBuffer[x++]=1;
  CmdBuffer[x++]=0;
  // handle
  CmdBuffer[x++]=Handle;
  // count laenge
  CmdBuffer[x++]=2;
  CmdBuffer[x++]=0;
  // count
  CmdBuffer[x++]=(uint8_t)(Count % 256);
  CmdBuffer[x++]=(uint8_t)(Count / 256);
  // size laenge
  CmdBuffer[x++]=2;
  CmdBuffer[x++]=0;
  // size
  CmdBuffer[x++]=(uint8_t)(Size % 256);
  CmdBuffer[x++]=(uint8_t)(Size / 256);

  // data laenge
  y=Size*Count;
  CmdBuffer[x++]=y % 256;
  CmdBuffer[x++]=y / 256;

  // data
  while (y--)
  {
    CmdBuffer[x++]=*pData++;
  }

  x++;
  CmdBuffer[1]=(uint8_t)(x % 256);
  CmdBuffer[2]=(uint8_t)(x / 256);
  SendCommand();

  if (GetResult(STANDARD_TIMEOUT))
    return (Count*Size);
  else
    return 0;
}

///////////////////////////////////////////////////////////////////////////////
// WriteEnable_
///////////////////////////////////////////////////////////////////////////////
uint8_t WriteEnable_(void)
{
uint8_t x=0;
  CmdBuffer[x++]=WRITE_ENABLE;
  CmdBuffer[x++]=0;
  CmdBuffer[x++]=0;
  x++;
  CmdBuffer[1]=(uint8_t)(x % 256);
  CmdBuffer[2]=(uint8_t)(x / 256);
  SendCommand();
  return GetResult(STANDARD_TIMEOUT);
}

extern uint16_t FileRead_(uint8_t Handle, uint8_t *pData, uint16_t Count, uint16_t Size)
{

  CmdBuffer[ 0] = READ ;
  CmdBuffer[ 1] = 15 ;
  CmdBuffer[ 2] = 0 ;

  CmdBuffer[ 3] = 1 ;
  CmdBuffer[ 4] = 0 ;
  CmdBuffer[ 5] = Handle ;

  CmdBuffer[ 6] = 2 ;
  CmdBuffer[ 7] = 0 ;
  CmdBuffer[ 8] = Count % 256 ; // length of data to read low byte  
  CmdBuffer[ 9] = Count / 256 ; // length of data to read high byte 

  CmdBuffer[10] = 2;                      // length of size low byte  
  CmdBuffer[11] = 0;                      // length of size high byte 
  CmdBuffer[12] = Size % 256 ;            // size low byte            
  CmdBuffer[13] = Size / 256 ;            // size high byte           

  SendCommand() ;

  ReadSerial ( pData, Count ) ;

  return GetResult(STANDARD_TIMEOUT) ;

  // Die Funktion FileRead_ gibt als Rckgabewert die Anzahl der gelesen Zeichen
  // zurck. Daher muss die berprfung der Datenblockchecksumme durch die Funktion
  // erfolgen die FileRead_ aufruft
}

uint8_t FileSeek_(uint8_t Handle, uint32_t Position, uint8_t Whence)
{
uint16_t x=0;
  CmdBuffer[x++]=SEEK;
  // platzhalter gesamtlaenge
  CmdBuffer[x++]=0;
  CmdBuffer[x++]=0;
  // handle laenge
  CmdBuffer[x++]=1;
  CmdBuffer[x++]=0;
  // handle
  CmdBuffer[x++]=Handle;
  // position laenge
  CmdBuffer[x++]=4;
  CmdBuffer[x++]=0;
  // position
  CmdBuffer[x++]=(uint8_t)(Position       & 0xFF);
  CmdBuffer[x++]=(uint8_t)(Position >>  8 & 0xFF);
  CmdBuffer[x++]=(uint8_t)(Position >> 16 & 0xFF);
  CmdBuffer[x++]=(uint8_t)(Position >> 24 & 0xFF);
  // size whence
  CmdBuffer[x++]=2;
  CmdBuffer[x++]=0;
  // whence
  CmdBuffer[x++]=(uint8_t)(Whence % 256);
  CmdBuffer[x++]=(uint8_t)(Whence / 256);
  // gesamt laenge absolut plus crc
  x++;
  CmdBuffer[1]=(uint8_t)(x % 256);
  CmdBuffer[2]=(uint8_t)(x / 256);
  SendCommand();
  return GetResult(STANDARD_TIMEOUT);
}


