#include <REG8252.H>
#include "double.h"

#define binmax 0x40
unsigned char idata bufin [binmax] ;	// Eingabe-Puffer der seriellen Schnittstelle
unsigned char data  binread ;			// Lese-Zeiger Eingabe-Puffer
unsigned char data  binwrite ;			// Schreib-Zeiger Eingabe-Puffer

#define boutmax 0x10
unsigned char idata bufout[boutmax] ;	// Ausgabe-Puffer der seriellen Schnittstelle
unsigned char data  boutread ;			// Lese-Zeiger Ausgabe-Puffer
unsigned char data  boutwrite ;			// Schreib-Zeiger Ausgabe-Puffer

bit TIEnable ;     						// Wenn 1, dann bertragung aktiv, sonst inaktiv

//
// Interrupt-Funktion zum Empfangen/Senden von Zeichen ber die serielle Schnittstelle
//
void serial (void) interrupt 4 using 2
{

 if (RI) 
  {
   RI = 0 ;
   bufin[binwrite] = SBUF ;
   ++binwrite ;
   if (binwrite == binmax) 
     binwrite = 0 ;
  }

 if (TI)
  {
   TI = 0 ;
   if (boutread != boutwrite) 	// Wenn noch Zeichen im Puffer sind, dann bertragen, sonst bertragung deaktivieren
    {
     SBUF = bufout[boutread] ;
     ++boutread ;
     if (boutread == boutmax)
       boutread = 0 ;
    }
   else
    {
	 TIEnable = 0 ;
    }
  }
}

//
//
//
void resetbuffer(void)
{
 boutwrite = 0 ;
 boutread  = 0 ;
 binwrite  = 0 ;
 binread   = 0 ;
}

/*
  Aufgabe: Initialisierung der seriellen Schnittstelle.
*/
void resetserial (void)
{
 SCON  = 0x50;			            	// SCON: Mode 1, 8-bit UART
 TMOD  = (TMOD & 0x0F) | 0x20;			// TMOD: Timer 1, Mode 2, 8-bit Reload 		
 TH1   = 0xfA;							// TH1:  Reload Value 9600 Bit/s @ 11 MHZ
 PCON |= 0x80 ;							//       Double Speed
 TR1   = 1;								// TR1:  Timer 1 Run 				
 resetbuffer() ;						// Puffer lschen 
 TIEnable = 0 ;							// bertragung inaktiv
 ES       = 1 ;							// Serieller Interrupt ein
}

/*
  Aufgabe: Ein Zeichen ber die serielle Schnittstelle senden
*/
unsigned char putch (unsigned char c) 
{

 while (((boutwrite+1) % boutmax) == boutread)	// warte bis Puffer frei ist
   ;

 ES  = 0 ;
 bufout[boutwrite] = c ;	// Zeichen in den Ausgangspuffer schreiben
 ++boutwrite ;				// Pufferzeiger auf die nchste Position vorrcken
 if (boutwrite == boutmax)
   boutwrite = 0 ;
 if (!TIEnable)				// Wenn die bertragung inaktiv ist, dann Flag TIEnable setzen und mit TI aktivieren
  {
   TIEnable = 1 ;
   TI  		= 1 ;
  } 

 ES = 1 ;
 return (c) ;
}

/*
  Aufgabe: Ein Zeichen aus dem Puffer der seriellen Schnittstelle lesen
*/
unsigned char getch(void)
{
 unsigned int timeout ;
 unsigned char c ;

 timeout = STANDARD_TIMEOUT ;

 while ((binread == binwrite) && ( timeout ))	// Zeichen eingegangen oder Timeout?
  {
   MsDelay ( 1 ) ;
   --timeout ;
  } ;

 if (timeout == 0)												// Timeout aufgetreten?
  {
   c = 0 ;													// Eine 0 als Zeichen zurckgeben
  }
 else
  {
   c = bufin[binread] ;										// Kein Timeout also Zeichen eingetroffen
   ++binread ;												// Zeichen aus dem Puffer lesen
   if (binread == binmax)									// Puffer-Zeiger vorrcken
    binread = 0 ;
  }

 return (c) ;
}


//
// Anzahl der Zeichen im Empfangspuffer
//
unsigned char GetInQueue ( void )
{
 if ( binwrite >= binread )
   return ( binwrite - binread ) ;
 else
   return ( binwrite - binread + 64 ) ;
}


//
// ReadSerial
//
unsigned char ReadSerial(unsigned char *pBuffer, unsigned char  BytesToRead)
{
 unsigned char BytesAreRead=0;

 while ( BytesToRead )
  {
   pBuffer[BytesAreRead] = getch() ;
   ++BytesAreRead ;
   --BytesToRead ;
  }

 return BytesAreRead;
}


////////////////////////////////////////////////////////////////////////////////
// WriteSerial
////////////////////////////////////////////////////////////////////////////////
unsigned char WriteSerial(unsigned char *pBuffer, unsigned char  BytesToWrite)
{
 unsigned char BytesAreWritten=0;

 while ( BytesToWrite )
  {
   putch( pBuffer[BytesAreWritten] ) ;
   ++BytesAreWritten ;
   --BytesToWrite ;
  }

 return BytesAreWritten;
}


////////////////////////////////////////////////////////////////////////////////
// FlushSerialBuffer
////////////////////////////////////////////////////////////////////////////////
void FlushSerialBuffer(void)
{
}





